#ifdef APSTUDIO_INVOKED
	#error This file is not editable by Microsoft Visual C++
#endif //APSTUDIO_INVOKED

#include "Winver.h"

/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 0,3,2,1
 PRODUCTVERSION 0,0,0,0
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x2L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "Comments", "Distributed under the terms of the GNU Lesser General Public License (LGPL) version 2.1. See COPYING for details.\0"
            VALUE "CompanyName", "Bastiaan Bakker, LifeLine Networks bv \0"
            VALUE "ContributingAuthors", "Cedric Le Goater, Marc Welz, others. See documentation for complete list\0"     
            VALUE "FileDescription", "Log library for C++ DLL\0"
            VALUE "FileVersion", "0.4.3B\0"
            VALUE "InternalName", "log4cpp\0"
            VALUE "LegalCopyright", "Copyright © 2000-2002 the Log4cpp Project, others\0"
            VALUE "LegalTrademarks", "\0"
#ifdef _DEBUG
            VALUE "OriginalFilename", "log4cppD.dll\0"
#else
            VALUE "OriginalFilename", "log4cpp.dll\0"
#endif
            VALUE "PrivateBuild", "\0"
            VALUE "ProductName", "Log library for C++\0"
            VALUE "ProductVersion", "0.3.4B\0"
            VALUE "SpecialBuild", "\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END

// End of version info resource
